@@ -3,14 +3,14 @@ drawing_type_instances.py
 """
 population = [
     {'Name': 'Starr class diagram'},
-    {'Name': 'xuml class diagram'},
+    {'Name': 'xUML class diagram'},
     {'Name': 'Shlaer-Mellor class diagram'},
 
-    {'Name': 'xuml state machine diagram'},
+    {'Name': 'xUML state machine diagram'},
 
     {'Name': 'Starr class collaboration diagram'},
-    {'Name': 'xuml class collaboration diagram'},
+    {'Name': 'xUML class collaboration diagram'},
 
     {'Name': 'Starr domain diagram'},
-    {'Name': 'xuml domain diagram'},
+    {'Name': 'xUML domain diagram'},
 ]
\ No newline at end of file
